"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createEntityClass = exports.generateEntities = void 0;
const path_1 = __importDefault(require("path"));
const typescript_1 = __importDefault(require("typescript"));
const ts_helpers_1 = require("../../../../../../common/ts-helpers");
const entity_views_generation_1 = require("./entity-views-generation");
const import_utils_1 = require("../import-utils");
const constants_1 = require("../../../../../../common/constants");
const utils_1 = require("../../../../../../common/utils");
const model_utils_1 = require("./model-utils");
/**
 *
 * Generate TS entity classes from ProjectModel, write generated files to destDir
 *
 * @param projectModel project model entities generated from
 * @param destDir where created TS files should be placed, also need to compute correct imports in generated TS files
 * @param fs Yeoman MemFs editor
 * @return model context contains entity and enum maps with fqn as key
 */
function generateEntities(projectModel, destDir, fs) {
    const { entitiesMap, enumsMap } = model_utils_1.collectModelContext(projectModel);
    for (const [, entityInfo] of entitiesMap) {
        const { entity } = entityInfo;
        const ctx = {
            entitiesMap, entity, enumsMap, isBaseProjectEntity: entityInfo.isBaseProjectEntity
        };
        const { importInfos, classDeclaration } = createEntityClass(ctx);
        const includes = import_utils_1.createIncludes(importInfos, createImportInfo(entityInfo, ctx.isBaseProjectEntity));
        const views = entity_views_generation_1.createEntityViewTypes(entity, projectModel);
        const entityPath = !entityInfo.isBaseProjectEntity ? constants_1.ENTITIES_DIR : path_1.default.posix.join(constants_1.ENTITIES_DIR, constants_1.BASE_ENTITIES_DIR);
        fs.write(path_1.default.posix.join(destDir, entityPath, utils_1.getEntityModulePath(entity) + '.ts'), ts_helpers_1.renderTSNodes([...includes, classDeclaration, ...views]));
    }
    if (enumsMap.size > 0)
        fs.write(path_1.default.posix.join(destDir, constants_1.ENUMS_DIR, constants_1.ENUMS_FILE + '.ts'), ts_helpers_1.renderTSNodes([...enumsMap.values()], '\n\n'));
    return { entitiesMap, enumsMap };
}
exports.generateEntities = generateEntities;
function createEntityClass(ctx) {
    const heritageInfo = createEntityClassHeritage(ctx);
    const importInfos = [];
    if (heritageInfo.parentEntity) {
        importInfos.push(createImportInfo(heritageInfo.parentEntity, ctx.isBaseProjectEntity));
    }
    const classMembersInfo = createEntityClassMembers(ctx);
    if (classMembersInfo.importInfos) {
        importInfos.push(...classMembersInfo.importInfos);
    }
    return {
        classDeclaration: typescript_1.default.createClassDeclaration(undefined, [
            typescript_1.default.createToken(typescript_1.default.SyntaxKind.ExportKeyword)
        ], ctx.entity.className, undefined, heritageInfo.heritageClauses, classMembersInfo.classMembers),
        importInfos
    };
}
exports.createEntityClass = createEntityClass;
function createEntityClassHeritage(ctx) {
    const { entity, entitiesMap } = ctx;
    if (!entity.parentClassName || !entity.parentPackage) {
        return { heritageClauses: [] };
    }
    if (entity.parentClassName === 'AbstractSearchFolder') { // todo AbstractSearchFolder does not have name (?)
        return { heritageClauses: [] };
    }
    const parentEntityInfo = entitiesMap.get(entity.parentPackage + '.' + entity.parentClassName);
    if (!parentEntityInfo) {
        return { heritageClauses: [] };
    }
    return {
        heritageClauses: [typescript_1.default.createHeritageClause(typescript_1.default.SyntaxKind.ExtendsKeyword, [
                typescript_1.default.createExpressionWithTypeArguments([], typescript_1.default.createIdentifier(entity.parentClassName))
            ])],
        parentEntity: parentEntityInfo
    };
}
function createEntityClassMembers(ctx) {
    const { entity } = ctx;
    const basicClassMembers = entity.name != null
        ? [
            typescript_1.default.createProperty(undefined, [typescript_1.default.createToken(typescript_1.default.SyntaxKind.StaticKeyword)], 'NAME', undefined, undefined, typescript_1.default.createLiteral(entity.name))
        ]
        : [];
    if (!entity.attributes) {
        return { classMembers: basicClassMembers, importInfos: [] };
    }
    const importInfos = [];
    const allClassMembers = [...basicClassMembers, ...entity.attributes
            .filter(entityAttr => {
            if (entity.idAttributeName == null || entity.idAttributeName === 'id' || entityAttr.name === entity.idAttributeName) {
                return true;
            }
            else {
                // An edge case when we have a non-ID string attribute named "id", and a differently named ID attribute.
                // We don't include the former to the TS class, so that we don't end up with two properties named "id".
                return entityAttr.name !== 'id';
            }
        })
            .map(entityAttr => {
            const attributeTypeInfo = createAttributeType(entityAttr, ctx);
            if (attributeTypeInfo.importInfo) {
                importInfos.push(attributeTypeInfo.importInfo);
            }
            const idAttrName = ctx.entity.idAttributeName ?? 'id';
            const typeNode = isIdAttr(entityAttr.name, idAttrName)
                ? getIdTypeNode(entityAttr.mappingType)
                : createUnionWithNull(attributeTypeInfo.node);
            // REST API puts the id into the property "id" regardless of the actual attribute name.
            const attrName = entityAttr.name === idAttrName ? 'id' : entityAttr.name;
            return typescript_1.default.createProperty(undefined, undefined, attrName, typescript_1.default.createToken(typescript_1.default.SyntaxKind.QuestionToken), typeNode, undefined);
        })];
    return { classMembers: allClassMembers, importInfos };
}
function createUnionWithNull(node) {
    return typescript_1.default.factory.createUnionTypeNode([
        node,
        typescript_1.default.factory.createLiteralTypeNode(typescript_1.default.factory.createToken(typescript_1.default.SyntaxKind.NullKeyword))
    ]);
}
/**
 * TS attribute type - could be primitive, enum or entity class (single or array, depends on relation cardinality)
 *
 * @param entityAttr attribute which type should be computed
 * @param ctx context of attribute owner class
 */
function createAttributeType(entityAttr, ctx) {
    let node;
    let refEntity;
    let enumDeclaration;
    const { mappingType } = entityAttr;
    // primitive
    if (mappingType === 'DATATYPE') {
        switch (entityAttr.type.fqn) {
            case 'java.lang.Boolean':
                node = typescript_1.default.createKeywordTypeNode(typescript_1.default.SyntaxKind.BooleanKeyword);
                break;
            case 'java.lang.Integer':
                node = typescript_1.default.createKeywordTypeNode(typescript_1.default.SyntaxKind.NumberKeyword);
                break;
            case 'java.lang.String':
                node = typescript_1.default.createKeywordTypeNode(typescript_1.default.SyntaxKind.StringKeyword);
                break;
            default:
                node = typescript_1.default.createKeywordTypeNode(typescript_1.default.SyntaxKind.AnyKeyword);
                break;
        }
    }
    //objects
    if (mappingType === 'ASSOCIATION' || mappingType === 'COMPOSITION') {
        refEntity = ctx.entitiesMap.get(entityAttr.type.fqn);
        if (refEntity) {
            switch (entityAttr.cardinality) {
                case 'MANY_TO_MANY':
                case 'ONE_TO_MANY':
                    node = typescript_1.default.createArrayTypeNode(typescript_1.default.createTypeReferenceNode(entityAttr.type.className, undefined));
                    break;
                case 'ONE_TO_ONE':
                case 'MANY_TO_ONE':
                default:
                    node = typescript_1.default.createTypeReferenceNode(entityAttr.type.className, undefined);
                    break;
            }
        }
    }
    //enums
    if (mappingType == 'ENUM') {
        enumDeclaration = ctx.enumsMap.get(entityAttr.type.fqn);
        if (enumDeclaration) {
            node = typescript_1.default.createTypeReferenceNode(enumDeclaration.name.text, undefined);
        }
    }
    if (!node) {
        node = typescript_1.default.createKeywordTypeNode(typescript_1.default.SyntaxKind.AnyKeyword);
    }
    let importInfo = undefined;
    if (refEntity && refEntity.entity)
        importInfo = createImportInfo(refEntity, ctx.isBaseProjectEntity);
    if (enumDeclaration)
        importInfo = import_utils_1.enumImportInfo(enumDeclaration, ctx.isBaseProjectEntity ? '../..' : '..');
    return {
        node,
        importInfo
    };
}
function createImportInfo(importedEntity, isCurrentEntityBase) {
    if (isCurrentEntityBase && importedEntity.isBaseProjectEntity) {
        //we don't need BASE prefix if current entity and imported base entity in same base/ directory
        return {
            importPath: './' + utils_1.getEntityModulePath(importedEntity.entity),
            className: importedEntity.entity.className
        };
    }
    else {
        return import_utils_1.entityImportInfo(importedEntity);
    }
}
function isIdAttr(entityAttrName, idAttrName) {
    return entityAttrName === idAttrName;
}
function getIdTypeNode(idMappingTime) {
    return idMappingTime == "EMBEDDED"
        ? typescript_1.default.createKeywordTypeNode(typescript_1.default.SyntaxKind.ObjectKeyword)
        : typescript_1.default.createKeywordTypeNode(typescript_1.default.SyntaxKind.StringKeyword);
}
//# sourceMappingURL=entities-generation.js.map