"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.deriveGraphQLEditorModel = exports.deriveEntityDetailsTemplateModel = void 0;
const graphql_1 = require("graphql");
const util_1 = require("../../../building-blocks/stages/template-model/pieces/util");
const graphql_tag_1 = __importDefault(require("graphql-tag"));
const ScreenTemplateModel_1 = require("../../../building-blocks/stages/template-model/pieces/amplicode/ScreenTemplateModel");
const BaseTemplateModel_1 = require("../../../building-blocks/stages/template-model/pieces/amplicode/BaseTemplateModel");
const getEntityAttributes_1 = require("../../../building-blocks/stages/template-model/pieces/graphql-utils/getEntityAttributes");
const getEntityName_1 = require("../../../building-blocks/stages/template-model/pieces/graphql-utils/getEntityName");
const getTopFieldName_1 = require("../../../building-blocks/stages/template-model/pieces/graphql-utils/getTopFieldName");
const deriveEntityDetailsTemplateModel = async (options, answers, schema) => {
    const { query: queryString, mutation: mutationString, idField = 'id', refetchQueryName } = answers;
    const queryNode = graphql_tag_1.default(queryString);
    const mutationNode = mutationString != null
        ? graphql_tag_1.default(mutationString)
        : undefined;
    return {
        ...BaseTemplateModel_1.baseTemplateModel,
        ...util_1.templateUtilities,
        ...ScreenTemplateModel_1.deriveScreenTemplateModel(options, answers, schema),
        ...deriveGraphQLEditorModel(queryNode, idField, schema, mutationNode),
        // TODO problem with $id: String = "", quotation marks get messed up
        queryString,
        mutationString,
        idField,
        refetchQueryName
    };
};
exports.deriveEntityDetailsTemplateModel = deriveEntityDetailsTemplateModel;
function deriveGraphQLEditorModel(queryNode, idField, schema, mutationNode) {
    if (schema == null) {
        throw new Error('Schema is required for this generator');
    }
    const queryName = getTopFieldName_1.getTopFieldName(queryNode);
    const entityName = getEntityName_1.getEntityName(queryName, schema);
    if (mutationNode == null) {
        const readOnlyAttributes = getEntityAttributes_1.getEntityAttributes(queryNode, schema, idField);
        return {
            queryName,
            attributes: readOnlyAttributes,
            entityName
        };
    }
    const operationDefinition = mutationNode.definitions[0];
    if (!('variableDefinitions' in operationDefinition) || operationDefinition.variableDefinitions == null) {
        throw new Error('Variable definitions not found in mutation');
    }
    const inputVariableName = operationDefinition.variableDefinitions[0].variable.name.value;
    const inputType = operationDefinition.variableDefinitions[0].type;
    let inputTypeName;
    if ('name' in inputType) {
        inputTypeName = inputType.name.value;
    }
    if ('type' in inputType && 'name' in inputType.type) {
        inputTypeName = inputType.type.name.value;
    }
    if (inputTypeName == null) {
        throw new Error('Input type name not found');
    }
    const mutationName = getTopFieldName_1.getTopFieldName(mutationNode);
    const queryType = schema.getQueryType();
    if (queryType == null) {
        throw new Error('Query type not found');
    }
    const outputType = queryType.getFields()[queryName].type;
    if (!('name' in outputType)) {
        throw new Error('Output type name not found');
    }
    const typeMap = schema.getTypeMap();
    if (typeMap == null) {
        throw new Error('Type map not found');
    }
    const namedType = typeMap[inputTypeName];
    if (namedType == null
        || namedType instanceof graphql_1.GraphQLScalarType
        || namedType instanceof graphql_1.GraphQLUnionType
        || namedType instanceof graphql_1.GraphQLEnumType) {
        throw new Error(`Unexpected type ${namedType}`);
    }
    let hasStringScalars = false;
    let hasIntScalars = false;
    let hasFloatScalars = false;
    const hasIDScalars = false;
    let hasBooleanScalars = false;
    let hasEnumScalars = false;
    let hasUnknownCustomScalars = false;
    let hasRelationFields = false;
    let hasDateScalars = false;
    let hasTimeScalars = false;
    // We take attributes from query, otherwise it won't be possible to pair the entity type from editor with entity type from list
    const attributes = getEntityAttributes_1.getEntityAttributes(queryNode, schema, idField);
    // We need to take some info from mutation input and add it to `attributes`
    Object.values(namedType.getFields()).map((field) => {
        const attr = attributes.find(a => a.name === field.name);
        if (attr == null) {
            return;
        }
        switch (attr.type) {
            case 'Int':
            case 'ID':
            case 'BigInteger':
            case 'BigDecimal':
            case 'Long':
                hasIntScalars = true;
                break;
            case 'Float':
                hasFloatScalars = true;
                break;
            case 'String':
                hasStringScalars = true;
                break;
            case 'Boolean':
                hasBooleanScalars = true;
                break;
            case 'Date':
            case 'DateTime':
            case 'LocalDateTime':
            case 'Timestamp':
                hasDateScalars = true;
                break;
            case 'Time':
            case 'LocalTime':
                hasTimeScalars = true;
                break;
            default:
                if (field.type instanceof graphql_1.GraphQLEnumType) {
                    attr.enumOptions = field.type.getValues();
                    hasEnumScalars = true;
                    break;
                }
                if (field.type instanceof graphql_1.GraphQLInputObjectType) {
                    attr.isRelationField = true;
                    hasRelationFields = true;
                    break;
                }
                if (field.type instanceof graphql_1.GraphQLScalarType) {
                    hasUnknownCustomScalars = true;
                }
        }
    });
    return {
        queryName,
        mutationName,
        attributes,
        entityName,
        hasStringScalars,
        hasIntScalars,
        hasFloatScalars,
        hasIDScalars,
        hasBooleanScalars,
        hasEnumScalars,
        hasUnknownCustomScalars,
        hasRelationFields,
        hasDateScalars,
        hasTimeScalars,
        inputVariableName,
        inputTypeName
    };
}
exports.deriveGraphQLEditorModel = deriveGraphQLEditorModel;
//# sourceMappingURL=template-model.js.map