"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const commons_1 = require("../../commons");
const chai_1 = require("chai");
const test_commons_1 = require("../../../test-commons");
const queries_1 = require("../common/queries");
const utils_1 = require("../../../../common/utils");
const GENERATOR_DIR = path_1.default.join(commons_1.GENERATORS_DIR, 'entity-management');
const DEST_DIR = path_1.default.join(commons_1.GENERATORS_DEST_DIR, 'entity-management');
describe('codegen readonly cards', () => {
    beforeEach(async () => await commons_1.cleanup(DEST_DIR));
    it('should generate readonly cards screen - Owner', async () => {
        const answers = {
            listComponentName: 'ReadOnlyOwnerCards',
            itemComponentName: 'ReadOnlyOwnerCardsDetails',
            route: 'read-only-owner-cards',
            shouldAddToMenu: true,
            listQuery: queries_1.ownerListQuery,
            detailsQuery: queries_1.ownerDetailsQuery,
            mode: 'view with details',
            type: 'cards'
        };
        const componentPath = path_1.default.join(DEST_DIR, 'ReadOnlyOwnerCards.tsx');
        const detailsComponentPath = path_1.default.join(DEST_DIR, 'ReadOnlyOwnerCardsDetails.tsx');
        await commons_1.generate(GENERATOR_DIR, commons_1.opts(DEST_DIR, answers, [commons_1.SCHEMA_PATH]));
        const expectTag = `
    <Space direction="vertical" className="card-space">
      <Cards items={items} loading={loading} error={error} />
      {/* <Pagination /> - in future */}
    </Space>`;
        const componentFile = fs_1.default.readFileSync(componentPath, 'utf-8');
        chai_1.expect(componentFile).to.contain('export function ReadOnlyOwnerCards() ');
        chai_1.expect(componentFile).to.contain('query Get_Owner_List {');
        test_commons_1.expectFileContainsIgnoreSpace(componentFile, expectTag);
        const detailsComponentFile = fs_1.default.readFileSync(detailsComponentPath, 'utf-8');
        chai_1.expect(detailsComponentFile).to.contain('query Get_Owner($id: BigInteger) {');
        ['First Name', 'Last Name', 'City', 'Address', 'Telephone', 'Email'].forEach(label => {
            const field = utils_1.unCapitalizeFirst(label.replace(" ", ""));
            const valueWithLabel = `
      <ValueWithLabel
        key="${field}"
        label="${label}"
        value={item.${field} ?? undefined}
      />`;
            test_commons_1.expectFileContainsIgnoreSpace(componentFile, valueWithLabel);
            const descriptions = `
        <Descriptions.Item label={<strong>${label}</strong>}>
          {item.${field} ?? undefined}
        </Descriptions.Item>`;
            test_commons_1.expectFileContainsIgnoreSpace(detailsComponentFile, descriptions);
        });
    });
    it('should generate readonly cards screen - Pet', async () => {
        const answers = {
            listComponentName: 'ReadOnlyPetCards',
            itemComponentName: 'ReadOnlyPetCardsDetails',
            route: 'read-only-pet-cards',
            shouldAddToMenu: true,
            listQuery: queries_1.petListQuery,
            detailsQuery: queries_1.petDetailsQuery,
            mode: 'view with details',
            type: 'cards'
        };
        const componentPath = path_1.default.join(DEST_DIR, 'ReadOnlyPetCards.tsx');
        const detailsComponentPath = path_1.default.join(DEST_DIR, 'ReadOnlyPetCardsDetails.tsx');
        await commons_1.generate(GENERATOR_DIR, commons_1.opts(DEST_DIR, answers, [commons_1.SCHEMA_PATH]));
        const expectTag = `
    <Space direction="vertical" className="card-space">
      <Cards items={items} loading={loading} error={error} />
      {/* <Pagination /> - in future */}
    </Space>`;
        const componentFile = fs_1.default.readFileSync(componentPath, 'utf-8');
        chai_1.expect(componentFile).to.contain('export function ReadOnlyPetCards() ');
        chai_1.expect(componentFile).to.contain('query Get_Pet_List {');
        test_commons_1.expectFileContainsIgnoreSpace(componentFile, expectTag);
        const detailsComponentFile = fs_1.default.readFileSync(detailsComponentPath, 'utf-8');
        chai_1.expect(detailsComponentFile).to.contain('query Get_Pet($id: BigInteger) {');
        // fields
        ['Identification Number'].forEach(label => {
            const field = utils_1.unCapitalizeFirst(label.replace(" ", ""));
            const valueWithLabel = `
      <ValueWithLabel
        key="${field}"
        label="${label}"
        value={item.${field} ?? undefined}
      />`;
            test_commons_1.expectFileContainsIgnoreSpace(componentFile, valueWithLabel);
            const descriptions = `
        <Descriptions.Item label={<strong>${label}</strong>}>
          {item.${field} ?? undefined}
        </Descriptions.Item>`;
            test_commons_1.expectFileContainsIgnoreSpace(detailsComponentFile, descriptions);
        });
        // Date fields
        ['Birth Date'].forEach(label => {
            const field = utils_1.unCapitalizeFirst(label.replace(" ", ""));
            const valueWithLabel = `
      <ValueWithLabel
        key="${field}"
        label="${label}"
        value={item.${field}?.format("LL") ?? undefined}
      />`;
            test_commons_1.expectFileContainsIgnoreSpace(componentFile, valueWithLabel);
            const descriptions = `
        <Descriptions.Item label={<strong>${label}</strong>}>
          {item.${field}?.format("LL") ?? undefined}
        </Descriptions.Item>`;
            test_commons_1.expectFileContainsIgnoreSpace(detailsComponentFile, descriptions);
        });
        // relation fields
        ['Owner'].forEach(label => {
            const field = utils_1.unCapitalizeFirst(label.replace(" ", ""));
            const valueWithLabel = `
      <ValueWithLabel
        key="${field}"
        label="${label}"
        value={get${utils_1.capitalizeFirst(field)}DTODisplayName(item.${field} ?? undefined)}
      />`;
            test_commons_1.expectFileContainsIgnoreSpace(componentFile, valueWithLabel);
            const descriptions = `
        <Descriptions.Item label={<strong>${label}</strong>}>
          {get${utils_1.capitalizeFirst(field)}DTODisplayName(item.${field} ?? undefined)}
        </Descriptions.Item>`;
            test_commons_1.expectFileContainsIgnoreSpace(detailsComponentFile, descriptions);
        });
        // enum PetType
        const valueWithLabel = `
      <ValueWithLabel
        key="type"
        label="Type"
        value={getPetTypeDTODisplayName(item.type ?? undefined)}
      />`;
        test_commons_1.expectFileContainsIgnoreSpace(componentFile, valueWithLabel);
        const descriptions = `
        <Descriptions.Item label={<strong>Type</strong>}>
          {getPetTypeDTODisplayName(item.type ?? undefined)}
        </Descriptions.Item>`;
        test_commons_1.expectFileContainsIgnoreSpace(detailsComponentFile, descriptions);
    });
    it('should generate readonly cards screen - ScalarsTestEntity', async () => {
        const answers = {
            listComponentName: 'ReadOnlyScalarsCards',
            itemComponentName: 'ReadOnlyScalarsCardsDetails',
            route: 'read-only-scalars-cards',
            shouldAddToMenu: true,
            listQuery: queries_1.scalarsListQuery,
            detailsQuery: queries_1.scalarsDetailsQuery,
            mode: 'view with details',
            type: 'cards'
        };
        const componentPath = path_1.default.join(DEST_DIR, 'ReadOnlyScalarsCards.tsx');
        const detailsComponentPath = path_1.default.join(DEST_DIR, 'ReadOnlyScalarsCardsDetails.tsx');
        await commons_1.generate(GENERATOR_DIR, commons_1.opts(DEST_DIR, answers, [commons_1.SCHEMA_PATH]));
        const componentFile = fs_1.default.readFileSync(componentPath, 'utf-8');
        chai_1.expect(componentFile).to.contain('export function ReadOnlyScalarsCards() ');
        chai_1.expect(componentFile).to.contain('query Get_Scalars_List {');
        chai_1.expect((componentFile.match(/<ValueWithLabel/g) || []).length).to.eq(24);
        const detailsComponentFile = fs_1.default.readFileSync(detailsComponentPath, 'utf-8');
        chai_1.expect(detailsComponentFile).to.contain('query Get_Scalars($id: ID) {');
        chai_1.expect((detailsComponentFile.match(/<Descriptions.Item/g) || []).length).to.eq(24);
        test_commons_1.expectFileContainsIgnoreSpace(detailsComponentFile, `
    <Descriptions.Item label={<strong>Url</strong>}>
          {item.url && (
            <a target="_blank" rel="noreferrer" href={item.url}>
              {item.url}
            </a>
          )}
        </Descriptions.Item>`);
    });
});
//# sourceMappingURL=readonly-cards.test.js.map